/*
** ipxfury.h - header for ipxfury pieces
**
** Keith Zaback (krz@wcug.wwu.edu) Apr 98
*/

#ifndef INC_IPXFURY_H
#define INC_IPXFURY_H

/* get config */
#include "config.h"

/* includes */
#include <stdio.h>
#include <termios.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>

/* gnulibc */
#ifdef USE_GNULIBC
#include <ioctl-types.h>
#endif

/*
** Ethernet is kinda system specific
*/
#ifdef FURY_ETHERNET

#ifdef FURY_LINUX
#ifdef USE_GNULIBC
#include <net/ethernet.h>
#else
#include <linux/if_ether.h>
#endif	/* GNULIBC */
#endif	/* FURY_LINUX */

#ifdef	FURY_BSD
#include <sys/time.h>
#include <sys/ioctl.h>
#include <net/bpf.h>
#endif

#ifdef FURY_BSD
/* nothing here */
#endif	/* FURY_BSD */

#endif /* FURY_ETHERNET */

/* include eveything, so others just need us */
#include "stats.h"
#include "furytypes.h"
#include "hash.h"
#include "hdrcache.h"
#include "interface.h"
#include "fury.h"
#include "events.h"
#include "pktcache.h"
#include "buffer.h"
#include "ipx.h"
#include "porttable.h"
#include "commands.h"
#include "ipaddr.h"
#include "compress.h"
#include "timer.h"

/* version & options info */
typedef struct stVER_OPT_INFO {
	int		major, minor, build;
	int		options;
} VER_OPT_INFO;

/* wonky print action */
#define PRINT_IPX_NETWORK	0x0001
#define PRINT_IPX_HOST		0x0002
#define PRINT_IPX_PORT		0x0004

/* some defines for the modem control lines */
#define FURY_MODEM_READY	(TIOCM_DTR | TIOCM_RTS | TIOCM_DSR | TIOCM_CTS)
#define FURY_MODEM_CARRIER	(TIOCM_CAR)

typedef struct stCONFIG {
	char		logfilename[128];	/* name of the log file */
	int		background;		/* are we backgrounded? */
} CONFIG;


/* hardware address table entries */
#define FURY_LOCATION_UNKNOWN	0x0001		/* ?? */
#define FURY_LOCATION_LOCAL	0x0002		/* local ethernet */
#define FURY_LOCATION_REMOTE	0x0004		/* across the bridge */
#define FURY_LOCATION_BROADCAST	0x0008		/* for the broadcast addr */

typedef struct stHWADDR_ENTRY {
	char		hwaddr[6];		/* hardware address */
	INTERFACE	*interface;		/* what interface connects us to them */
	
	/* starcraft optimization */
	int		sc_resend_counter[8];	/* one for each player */
} HWADDR_ENTRY;


/* defines for the status variable */
#define FSTATUS_PROMPT		0		/* at the command prompt */
#define FSTATUS_FURIOUS		1		/* in fury mode */
#define FSTATUS_RESETTING	2
#define FSTATUS_CONFIGURE	3		/* reading the config file */

typedef struct stGLOBALS {
	INTERFACE	interfaces[FURY_MAX_INTERFACES]; /* the interfaces */
	int		total_interfaces;	/* how many we have in the array */
	CONFIG		config;		/* configuration issues */
	HASHTABLE	hwaddr_table;	/* table of hardware addresses */
	FILE		*logfile;	/* ptr to log file */
	int		done;		/* when we should quit */
	int		status;		/* current status */
	struct timeval	now;		/* current time */
	EVENTLIST	*events;	/* our events */
	PORT_TABLE	watched_ports;	/* ipx ports that we actually want */
	GLOBAL_STATS	stats;
	int		verbose_level;	/* how much the fury speaks */
	int		ver_major;
	int		ver_minor;
	int		ver_build;
	int		alarm;
} GLOBALS;


/* protos */
/* ipxfury.c */
void			cleanup(int exit_code);
void			ipx_fury(void);

/* misc.c */
char			*print_ether_addr(unsigned char *addr);
void			print_ipx_frame(BYTE *ipxframe, int show_data);
void			print_ipx_addr(IPXADDR *addr, int flags);
void			print_time(struct tm *t);
char			*print_datatrans(int amt);
/* ether.c */
/* These are used even if there is no ethernet device */
int			ether_hash(HWADDR_ENTRY *entry);
int			ether_compare(HWADDR_ENTRY *e1, HWADDR_ENTRY *e2);
void			ether_hash_enum(int bucketno, HWADDR_ENTRY *entry);

#ifdef FURY_ETHERNET
int			ether_open(INTERFACE *edev);
int			ether_close(INTERFACE *edev);
int			ether_read(INTERFACE *edev);
int			ether_process(INTERFACE *edev);
int			ether_queue_ipx_frame(INTERFACE *from, INTERFACE *edev, BYTE *ipxframe);
int			ether_write(INTERFACE *edev);
int			ether_flush(INTERFACE *edev);
int			ether_dummy(INTERFACE *edev);
int			ether_event(INTERFACE *edev, int event);
int			ether_prepare_device(INTERFACE *edev);
int			ether_build_frame(int type, IPXHEADER *ipxheader, BYTE *buffer);
IPXHEADER		*ether_get_ipx(int type, BYTE *buffer);
int			ether_changeframe(INTERFACE *edev, int new_frame);
#endif /* FURY_ETHERNET */

/* serial.c */
int			serial_open(INTERFACE *ser);
int			serial_close(INTERFACE *ser);
int			serial_read(INTERFACE *ser);
int			serial_process(INTERFACE *ser);
int			serial_queue_ipx_frame(INTERFACE *from, INTERFACE *ser, BYTE *ipxframe);
int			serial_write(INTERFACE *ser);
int			serial_status(INTERFACE *ser, int *result);
int			serial_timeout(INTERFACE *ser);
int			serial_flush(INTERFACE *ser);
int			serial_prepare_device(INTERFACE *ser);
int			serial_dummy(INTERFACE *ser);

/* term.c */
void			fury_term(char *args);

/* config.c */
int			read_config(char *filename, CONFIG *cfg);
void			show_config(CONFIG *cfg);
void			cleanline(char *line);

/* log.c */
void			errormsg(int type, char *func, char *fmt, ...);
void			log(char *fmt, ...);
void			notice(char *fmt, ...);
char			*current_time(void);

/* interface.c */
void			interface_setup(void);
void			interfaces_info(void);
INTERFACE		*interface_find(int target, int type);
INTERFACE	 	*interface_find_name(char *iname);

/* fury.c */
int			fury_receive(INTERFACE *fdev, BYTE *data, int length);
void			fury_event(INTERFACE *iface, int event);
void			send_lcp(INTERFACE *iface, BYTE lcp_code, BYTE id, BYTE *data, int length);
void			send_fury_frame(INTERFACE *iface, int type, BYTE *data, int data_len);
int			send_fury_ack(INTERFACE *fdev, BYTE ack_type, BYTE *data, int data_len);
int			send_fury_command(INTERFACE *fdev, BYTE cmd_type, BYTE *data, int data_len);

/* fury_ipx.c */
int			process_ipx(INTERFACE *fdev, int type, BYTE *data, int data_length);
int			process_ipx_cache_ack(INTERFACE *fdev, BYTE *data, int data_length);
int			get_address_locations(INTERFACE *iface, IPXHEADER *ipxheader, HWADDR_ENTRY **src_address, HWADDR_ENTRY **dest_address);
int			process_hdrcache_clear(INTERFACE *fdev, BYTE *frame_data, int data_length);
int			process_pktcache_clear(INTERFACE *fdev, BYTE *frame_data, int data_length);
int			create_fury_ipx_frame(INTERFACE *fdev, BYTE *orig_ipxframe, BYTE *new_fury_frame, int *new_type, int *length);


/* hexnew.c */
void			fhexdump(FILE *out, unsigned char *data, int len);
void			hexdump(unsigned char *data, int len);

/* commands.c */
void			process_terminal_command(char *cmd);
void			command_reset(int send_reset);
void			show_stats(char *args);
void			show_hdrcache(char *args);
void			show_modem_info(char *args);
void			show_pktcache(char *args);
void			show_help(void);
void			command_changeports(int set, char *args);
void			command_reset(int send_reset);
void			show_stats(char *args);
void			udp_command(char *args);
void			command_verbose(char *args);
void			command_set(char *args);
void			command_options(char *args);

/* porttable.c */
PORT_TABLE		create_port_table(void);
void			free_port_table(PORT_TABLE *tab);
void			show_set_ports(PORT_TABLE tab);

/* stats.c */
void			show_recent_stats(void);
void			show_all_stats(void);

/* udp.c */
int			udp_dummy(INTERFACE *udp);
int			udp_open(INTERFACE *udp);
int			udp_close(INTERFACE *udp);
int			udp_read(INTERFACE *udp);
int			udp_process(INTERFACE *udp);
int			udp_queue_ipx_frame(INTERFACE *from, INTERFACE *udp, BYTE *ipxframe);
int			udp_write(INTERFACE *udp);
int			udp_status(INTERFACE *udp, int *result);
int			udp_timeout(INTERFACE *udp);
int			udp_flush(INTERFACE *udp);
int			udp_prepare_device(INTERFACE *udp);
int			udp_event(INTERFACE *udp, int event);
int			udp_ipxframe_processor(INTERFACE *fdev, BYTE *ipxframe);
void			udp_dump_hwtable(int num, HWIP_ENTRY *entry);
void			udp_dump_iptable(int num, IPADDR_ENTRY *entry);
int			udp_process_error(INTERFACE *udp);
void			udp_hdrcache_maint(int num, IPADDR_ENTRY *ip);
int			udp_connect_req(INTERFACE *udp, struct sockaddr_in *to);
void			udp_maint(INTERFACE *udp);

/* tokenize.c */
int			tokenize(char *str, int toks_to_get, char tok_char, char *tok_list[]);

/* compress.c */
int			compress_z(char *in, int i_size, char *out, int o_size);
int			uncompress_z(char *in, int i_size, char *out, int o_size);
int			compress_d(DIFF_ARGS *args);
int			uncompress_d(DIFF_ARGS *args);
int			compress_d2(DIFF_ARGS *args);
int			uncompress_d2(DIFF_ARGS *args);

/* console.c */
int			terminal_read(int fd);
int			init_console(int *fd);
void			exit_console(int fd);
void			draw_statusbar(char *text);

/* starcraft.c */
int			starcraft_filter(INTERFACE *edev, BYTE *ipx, HWADDR_ENTRY *source);

#endif
